import json


class SecretKeeper:
    class Module:
        pass

    def __init__(self) -> None:
        pass

    @staticmethod
    def read_json(path: str) -> dict:
        with open(path, 'r') as file:
            data = json.load(file)
        return data

    def add_module(self, name: str, settings_path: str):
        module = type(name, (self.Module,), self.read_json(settings_path))
        setattr(self, name, module)



from typing import Callable, TYPE_CHECKING

from selenium.webdriver.common.by import By
from selenium.webdriver.support.expected_conditions import presence_of_all_elements_located
from selenium.webdriver.support.wait import WebDriverWait

from .element import ExtendedWebElement
from .. import settings


if TYPE_CHECKING:
    from parsing_helper.pages.base_page import BasePage


class ExtendedWebElementCollection:
    def __init__(self, page: "BasePage", xpath: str) -> None:
        self.page = page
        self.driver = page.driver

        self.xpath = xpath
        self.wait = WebDriverWait(self.driver, settings.DEFAULT_TIMEOUT)
        self.elements: None | list[ExtendedWebElement] = None
        self.initialized: dict[Callable, bool] = {}

    def reset(self) -> None:
        """Сбрасывает состояние коллекции к первоначальному."""

        self.initialized = {}

    def __len__(self) -> int:
        self.init_if_necessary()
        return len(self.elements)

    def __getitem__(self, index):
        self.init_if_necessary()
        return self.elements[index]

    def __iter__(self):
        self.init_if_necessary()
        for element in self.elements:
            yield element

    def init_if_necessary(self, wait_condition: Callable = presence_of_all_elements_located) -> None:
        if wait_condition not in self.initialized or not self.initialized[wait_condition]:
            self.init(wait_condition)

    def init(self, wait_condition: Callable = presence_of_all_elements_located) -> None:
        """Находит элементы коллекции в DOM-структуре страницы."""

        elements_amount = len(self.wait.until(wait_condition((By.XPATH, self.xpath))))
        self.elements = [
            # xpath начинает нумерацию в коллекциях с 1
            ExtendedWebElement(self.page, f"{self.xpath}[{number}]") for number in range(1, elements_amount)
        ]
        self.initialized[wait_condition] = True



from typing import Callable, TYPE_CHECKING

from selenium.webdriver.common.by import By
from selenium.webdriver.remote.webelement import WebElement
from selenium.webdriver.support.expected_conditions import element_to_be_clickable, presence_of_element_located, \
    visibility_of_element_located
from selenium.webdriver.support.wait import WebDriverWait

from .. import settings


if TYPE_CHECKING:
    from parsing_helper.pages.base_page import BasePage


class ExtendedWebElement:
    class WaitCondition:
        CLICKABLE = element_to_be_clickable
        PRESENCE = presence_of_element_located
        VISIBLE = visibility_of_element_located

    def __init__(self, page: "BasePage", xpath: str) -> None:
        self.page = page
        self.driver = page.driver

        self.xpath = xpath
        self.wait = WebDriverWait(self.driver, settings.DEFAULT_TIMEOUT)
        self.initialized: dict[Callable, bool] = {}
        self._selenium_element: None | WebElement = None

    def reset(self) -> None:
        """Сбрасывает состояние объекта к первоначальному."""

        self.initialized = {}
        self._selenium_element = None

    def init_if_necessary(self, wait_condition: Callable = WaitCondition.PRESENCE) -> None:
        if wait_condition not in self.initialized or not self.initialized[wait_condition]:
            self.init(wait_condition)

    def init(self, wait_condition: Callable = WaitCondition.PRESENCE) -> None:
        """Находит элемент в DOM-структуре страницы."""

        self._selenium_element = self.wait.until(wait_condition((By.XPATH, self.xpath)))
        self.initialized[wait_condition] = True

    @property
    def selenium_element(self) -> WebElement:
        self.init_if_necessary()
        return self._selenium_element

    @property
    def text(self) -> str:
        self.init_if_necessary()
        return self.selenium_element.text

    def click(self) -> None:
        self.init(self.WaitCondition.CLICKABLE)
        return self.selenium_element.click()

    def send_keys(self, value: str) -> None:
        self.init_if_necessary()
        return self.selenium_element.send_keys(value)

    def get_attribute(self, name) -> str:
        self.init_if_necessary()
        return self.selenium_element.get_attribute(name)



import json

from setuptools import setup


PROJECT_DATA_PATH = "project_data.json"
PROJECT_REQUIREMENTS_PATH = "requirements.txt"

with open(PROJECT_DATA_PATH, 'r') as project_data_file:
    project_data = json.load(project_data_file)

with open(PROJECT_REQUIREMENTS_PATH, 'r') as project_requirements_file:
    requirements_list = ["selenium"]
    requirements = []
    for requirement in requirements_list:
        for project_requirement in project_requirements_file:
            project_requirement_clean = project_requirement.strip().replace("\x00", "")
            if requirement in project_requirement_clean:
                requirements.append(project_requirement_clean)

setup(
    author = "Radislav Vlasov",
    author_email = "radislavvlasov@gmail.com",
    name = project_data["package_name"],
    version = project_data["version"],
    install_requires = requirements,
    data_files = [("", [PROJECT_DATA_PATH, PROJECT_REQUIREMENTS_PATH])]
)



from selenium.webdriver import Chrome


class BasePage:
    # элементы url
    scheme: str
    domain: str
    path: str
    parameters = {}

    def __init__(self, driver: Chrome) -> None:
        self.driver = driver

    def open(self) -> None:
        self.driver.get(self.url)

    @property
    def url(self) -> str:
        return self.construct_url()

    def construct_url(self, scheme = None, domain = None, path = None, parameters = None) -> str:
        if scheme is None:
            scheme = self.scheme
        if domain is None:
            domain = self.domain
        if path is None:
            path = self.path
        if parameters is None:
            parameters = self.parameters

        url = f"{scheme}://{domain}/{path}"
        if parameters:
            url += '?'
            for parameter_name in parameters:
                url += f"{parameter_name}={parameters[parameter_name]}&"
            url = url[:-1]
        return url

    def scroll(self, x_offset, y_offset):
        self.driver.execute_script(f"window.scrollBy({x_offset}, {y_offset});")

    def scroll_up(self, offset):
        self.scroll(0, -offset)

    def scroll_down(self, offset):
        self.scroll(0, offset)
